#BEGIN COPYRIGHT
#*************************************************************************
#
# Licensed Materials - Property of IBM
# 5725-C94, 5725-C95, 5725-C96
# (C) Copyright IBM Corporation 2013. All Rights Reserved.
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#
#*************************************************************************
#END COPYRIGHT

import os.path, re
from java.lang import System

__file__ = "restServiceGatewayUpdate.py"

appName = 'REST Services Gateway'
cellName = System.getProperty("local.cell")

#------------------------------------------------------------------------------
# log either to profile upgrade framework or to console, depending on context
#------------------------------------------------------------------------------
def log(level, method, message):
  if globals().has_key("framework"):
    framework.log(level, __file__, method, message)
  else:
    print message
#enddef

#------------------------------------------------------------------------------
# Get the containment path for a given scope
#------------------------------------------------------------------------------
def getScopePath(nodeName, serverName, clusterName):
  if clusterName:
    configPath = "/Cell:" + cellName + "/ServerCluster:" + clusterName + "/"
  elif serverName:
    configPath = "/Cell:" + cellName + "/Node:" + nodeName + "/Server:" + serverName + "/"
  elif nodeName:
    configPath = "/Cell:" + cellName + "/Node:" + nodeName + "/"
  else:
    configPath = "/Cell:" + cellName + "/"
  #endif
  return configPath
#enddef

#------------------------------------------------------------------------------
# Get the containment path for a given scope
#------------------------------------------------------------------------------
def getObjectName(nodeName, serverName, clusterName):
  if clusterName:
    configPath = "WebSphere:cell=" + cellName + ",cluster=" + clusterName
  elif serverName:
    configPath = "WebSphere:cell=" + cellName + ",node=" + nodeName + ",server=" + serverName
  #endif
  return configPath
#enddef

#------------------------------------------------------------------------------
# Get the key / value pairs for a given task
#------------------------------------------------------------------------------
def getTaskInfo(uri, task, lines):
  options = []
  taskInfo = AdminUtilities.convertToList(AdminApp.view(uri, [task]))
  # the last <lines> lines contain the task fields (key / value pairs), e.g.:
  #   Web module:  BPM Help
  #   URI:  BPMHelp.war,WEB-INF/web.xml
  #   Virtual host:  null
  # caution: the key may be translated!
  pattern = re.compile("[^:]*:\s*(.*)")
  for line in taskInfo[-lines:]:
    matched = pattern.match(line)
    if matched:
      options += [matched.group(1)]
  #endfor
  return options
#enddef

def isWebServer(target):
	#target is "WebSphere:Cell=Cell01,Node=node01,Server=server01"
	configPath = None	
	targetList = target.split(":")[1].split(",")
	#Only check target that type is server, web server can not be clustered
	if len(targetList) == 3:
		configPath = "/Cell:" + targetList[0].split("=")[1] + "/Node:" + targetList[1].split("=")[1] + "/Server:" + targetList[2].split("=")[1] + "/"
		server = AdminConfig.getid(configPath)
		serverType = AdminConfig.showAttribute(server,"serverType")
		return serverType 
	else:
		return None	
#enddef
def updateRESTServicesProvider(appName, newAppName, nodeName, serverName, clusterName):
    targetContextID = ''
    if clusterName:
        targetContextID = 'cells:' + cellName + ':clusters:' + clusterName
    else:
        targetContextID = 'cells:' + cellName + ':nodes:' + nodeName + ":servers:" + serverName
    WBIRestDeploymentTargets = AdminConfig.getid('/Cell:/WBIRestConfiguration:/WBIRestEndpoints:/WBIRestDeploymentTarget:/').splitlines()
    for WBIRestDeploymentTarget in WBIRestDeploymentTargets:
        contextID = AdminConfig.showAttribute(WBIRestDeploymentTarget,'contextID')
        if contextID == targetContextID:
            webModule = AdminConfig.showAttribute(WBIRestDeploymentTarget,'webModules')[1:-1]
            webModuleName = AdminConfig.showAttribute(webModule,'name')
            webModuleAppName = AdminConfig.showAttribute(webModule,'applicationName')
            if webModuleName == 'rest.gateway.war' and webModuleAppName == appName:
                attr = '[[applicationName "' + newAppName + '"]]'
                AdminConfig.modify(webModule,attr)
            #endif
        #endif
    #endfor
    AdminConfig.save()
#enddef 
#------------------------------------------------------------------------------
# This function implements the profile upgrade framework definition that accepts
# an appPrefix, nodeName, serverName, and clusterName).  It will be called by the
# framework function updateDeploymentTargetsOf.  Either the nodeName/serverName
# pair will be specified or the clusterName parameter will be specified.
# The others will be None.
#------------------------------------------------------------------------------
def updateRESTServicesGateway(appName, nodeName, serverName, clusterName):
  thisTarget = getObjectName(nodeName, serverName, clusterName)
  log("info", "updateRESTServicesGateway", "Processing " + appName + " on " + thisTarget + "...")
  if not clusterName:
    configPath = getScopePath(nodeName, serverName, clusterName)
    server = AdminConfig.getid(configPath)
    if AdminConfig.showAttribute(server, "serverType") == "WEB_SERVER":
      log("info", "updateRESTServicesGateway", "Deployment target " + configPath + " is a Web server, skipping.")
      return
    #endif
  #endif
  
  # get the current (old) application settings
  targetMappings = getTaskInfo(appName, "-MapModulesToServers", 3)
  roleToUserMapping = getTaskInfo(appName, "-MapRolesToUsers", 8)
  vhostMapping = getTaskInfo(appName, "-MapWebModToVH", 3)
  contextRoot = getTaskInfo(appName, "-CtxRootForWebMod", 3)

  # remove the deployment target that is being updated from the old application   
  targets = targetMappings[2].split('+')
  log("info", "updateRESTServicesGateway", "Deploy targets:" + str(targets))
  targets.remove(thisTarget)
  #remove the deployment target that is web server
  targets = [target for target in targets if not isWebServer(target)]
  log("info", "updateRESTServicesGateway", "Deploy targets after removing web server:" + str(targets))
  
  if not targets:
    # if the old application isn't mapped to any more targets, remove it 
    log("info", "updateRESTServicesGateway", "AdminApp.uninstall('" + appName + "')")
    AdminApp.uninstall(appName)
  else:
    # if the old application is mapped to more targets, just remove the current target from its target mappings
    options = []
    targetMappings[2] = "+".join(targets)
    options += ["-MapModulesToServers", [ targetMappings ]]
    log("info", "updateRESTServicesGateway", "AdminApp.edit('" + appName + "', " + str(options) + ")")
    AdminApp.edit(appName, options)
  #endif

  # install a new version of the application on the deployment target that is being updated
  newAppName = appName + "_"
  if clusterName:
    newAppName += clusterName
  else:
    newAppName += nodeName + "_" + serverName
  #endif
  
  # use the same settings that the old application had, with the exception of the target mappings
  if not newAppName in AdminUtilities.convertToList(AdminApp.list()):
      options = ["-appname", newAppName, "-nodeployejb"]
      targetMappings[2] = thisTarget
      options += ["-MapModulesToServers", [ targetMappings ]]
      options += ["-MapRolesToUsers", [ roleToUserMapping ]]
      options += ["-MapWebModToVH", [ vhostMapping ]]
      options += ["-CtxRootForWebMod", [ contextRoot ]]
      earFile = os.path.join(System.getProperty("was.install.root"), "installableApps/RESTServiceGateway.ear")
      log("info", "updateRESTServicesGateway", "AdminApp.install('" + earFile + "', " + str(options) + ")")
      AdminApp.install(earFile, options)
    
  updateRESTServicesProvider(appName, newAppName, nodeName, serverName, clusterName)
  log("info", "updateRESTServicesGateway", "Finished processing " + appName + ".")
#enddef

#------------------------------------------------------------------------------
# Main
#------------------------------------------------------------------------------
log("info", "main", __file__ + " started...")
log("info", "main", "Updating REST Services Gateway application in profile " + System.getProperty("user.install.root"))

if globals().has_key("framework"): # script called from upgrade framework

  # Use the framework's "updateDeploymentTargetsOf" function to invoke
  # the above callable (function) for each deployment target where 
  # 'REST Services Gateway' is deployed
  updateDeploymentTargetsOf([appName], updateRESTServicesGateway)

else: # script called from command line

  nodeName = None
  serverName = None
  clusterName = None

  if len(sys.argv) == 1:
    clusterName = sys.argv[0]
  elif len(sys.argv) == 2:
    nodeName = sys.argv[0]
    serverName= sys.argv[1]
  else:
    log("info", "main", "Usage: wsadmin -conntype none -f " + __file__ + " ( ( <nodeName> <serverName> ) | <clusterName> )")
    log("info", "main", "       The name of the local node is '" + System.getProperty("local.node") + "'")
    sys.exit(-1)
  #endif

  configPath = getScopePath(nodeName, serverName, clusterName)

  if not AdminConfig.getid(configPath):
    log("error", "main", "Deployment target " + configPath + " does not exist, exiting")
    sys.exit(-1)

  updateRESTServicesGateway(appName, nodeName, serverName, clusterName)

#endif

changes = AdminConfig.queryChanges()
if changes.startswith("WASX7241I:"):
  # WASX7241I: There are no unsaved changes in this workspace.
  log("info", "main", "Nothing changed.")
else:
  log("info", "main", "Saving changes...")
  AdminConfig.save()
#endif

log("info", "main", __file__ + " finished.")
